# Generated by Django 3.0.4 on 2020-03-27 00:03

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('file', '0002_auto_20200315_1105'),
    ]

    operations = [
        migrations.CreateModel(
            name='BashHistory',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('dt', models.DateTimeField()),
                ('command', models.TextField()),
                ('file', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='file.File')),
            ],
            options={
                'db_table': 'bash_history',
            },
        ),
    ]
